# Copyright 2005-2006 Timothy Appnel, tim@appnel.com.

package MT::XPlugin;
use strict;

use vars qw( $VERSION );
$VERSION = 0.3;

use base qw( Class::Accessor::Fast MT::Plugin);

__PACKAGE__->mk_accessors(
    qw( key name author_name author_link plugin_link version
      config_link doc_link description envelope
      system_config_template blog_config_template configmgr_class )
);

{

    # MT requires all plugins to descend from MT::Plugin in order to be
    # listed in weblog plugin settings. Most unfortunate. Remove the cruft to
    # avoid issues.
    local $SIG{__WARN__} = sub { };
    *_getset                = sub { };
    *settings               = sub { };
    *config_vars            = sub { };
    *apply_default_settings = sub { };
    *set_config_value       = sub { };
    *get_config_hash        = sub { };
    *get_config_value       = sub { };

  # One benefit of having to inherit MT::Plugin is that this line is not needed.
  # *load_tmpl = \&MT::Plugin::load_tmpl;
}

sub new {
    my $class = shift;
    my ($self) = ref $_[0] ? @_ : {@_};
    bless $self, $class;
    $self->{configmgr_class} = $class . '::ConfigMgr'
      unless ($self->{configmgr_class} || $class eq 'MT::XPlugin');
    $self->{__cfgmgr} = {};
    $self->init;
    $self;
}

sub init { }

sub init_app {
    my $plex = $_[0]->get_lexicon(MT->current_language) or return;
    $_[0]->add_lexicon($plex);
}

sub init_request {
    $_[0]->{__cfgmgr} = {} if $_[0]->{__cfgmgr};
}

sub load_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    my $cfgmgr = $plugin->config_mgr($scope) || return;
    %$param = %{$cfgmgr->data};    # clone it.
    foreach my $key ($cfgmgr->config_keys) {
        next unless exists $param->{$key};
        my $value = $cfgmgr->$key;
        $param->{$key} = $value;
        next
          if (!defined $value or $value =~ m/\s/ or length($value) > 100)
          ;    # what about illegal characters in this context?
        $param->{$key . '_' . $value} = 1;
    }
}

sub save_config {
    my $plugin = shift;
    my ($param, $scope) = @_;
    my $cfgmgr = $plugin->config_mgr($scope) || return;
    map { $cfgmgr->$_($param->{$_}) }
      $cfgmgr->config_keys;    # what if param is undefined or does not exist???
    $cfgmgr->save_config or $plugin->error($cfgmgr->errstr);
    1;
}

sub reset_config {
    my $plugin = shift;
    my $cfgmgr = $plugin->config_mgr($_[0]) || return;
    my $id     = $cfgmgr->{__id};
    if ($cfgmgr && defined $id) {
        delete $plugin->{__cfgmgr}->{$id};
        $cfgmgr->reset_config;
    }
    1;
}

sub config_template {
    my $plugin = shift;
    my ($param, $scope) = @_;
    $scope ||= 'system';
    $scope =~ s/:.*//;
    my $tmpl = $plugin->{"${scope}_config_template"}
      || $plugin->{'config_template'};
    return undef unless $tmpl;
    ref $tmpl eq 'CODE' ? $tmpl->($plugin, @_)
      : $tmpl =~ /\s/ ? $tmpl
      : $plugin->load_tmpl($tmpl);
}

sub config_mgr {
    my $plugin = shift;
    my $scope = shift || 'system';
    my ($id) = $scope eq 'system' ? 0 : $scope =~ /:(\d+)$/;
    return $plugin->{__cfgmgr}->{$id} if $plugin->{__cfgmgr}->{$id};
    my $class = $plugin->{configmgr_class};
    eval "use $class;";
    return $plugin->error("Error loading $class: $@") if $@;
    my $cfgmgr = $class->read_config($id)
      || return $plugin->error($class->errstr);
    $plugin->{__cfgmgr}->{$id} = $cfgmgr;
    $cfgmgr;
}

{
    local $SIG{__WARN__} = sub { };
    *get_config_obj = \&config_mgr;    # play nice with CMS plugin listing.
}

#--- untested. experimental.

sub get_lexicon {
    my ($plugin, $lang) = @_;
    return if $lang eq 'en';
    my $class = ref($plugin) . '::L10N::' . $lang;
    eval "use $class;";
    return if $@;
    no strict 'refs';
    my $plex = *{$class . '::Lexicon'}{'HASH'};
    $plex;
}

sub add_lexicon {
    my ($class, $plex) = @_;
    no strict 'refs';
    my $lex = *{ref(MT->language_handle) . '::Lexicon'}{'HASH'};
    map { $lex->{$_} = $plex->{$_} } keys %$plex;
}

1;

__END__

sub has_lexicon { # phrase???
    no strict 'refs';
    my $lex = *{ref( MT->language_handle ).'::Lexicon'}{'HASH'};
    exists $lex->{$_[1]}
}
