# EnhancedEntryEditing - A plugin for Movable Type.
# Copyright (c) 2005-2007, Arvind Satyanarayan.

package MT::Plugin::EnhancedEntryEditing;

use 5.006;    # requires Perl 5.6.x
use MT 3.3;   # requires MT 3.2 or later

use base 'MT::Plugin';
our $VERSION = '1.2';

my $plugin;
MT->add_plugin($plugin = __PACKAGE__->new({
	name            => "EnhancedEntryEditing",
	version         => $VERSION,
	description     => "Adds the <a href=\"http://tinymce.moxiecode.com/\">tinyMCE</a> WYSIWYG editor to the Movable Type entry editing screen.",
	author_name     => "Arvind Satyanarayan",
	author_link     => "http://www.movalog.com/",
	plugin_link     => "http://plugins.movalog.com/enhancedentryediting/",
	doc_link        => "http://plugins.movalog.com/enhancedentryediting/manual",
    settings        => new MT::PluginSettings([
		['tinymce_config', { Default => q{
theme : "advanced",
plugins : "inlinepopups",
theme_advanced_blockformats : "p,h1,h2,h3,h4,h5,h6",
theme_advanced_buttons1 : "formatselect,bold,italic,underline,strikethrough,separator,justifyleft,justifycenter,justifyright,justifyfull,separator,bullist,numlist,outdent,indent,separator,undo,redo,separator,link,unlink,separator,image",
theme_advanced_buttons2 : "",
theme_advanced_buttons3 : "",
theme_advanced_toolbar_location : "top",
theme_advanced_toolbar_align : "left",
theme_advanced_statusbar_location : "bottom",
theme_advanced_path : false,
theme_advanced_resizing : true,
theme_advanced_resize_horizontal : false,
extended_valid_elements : "a[name|href|target|title|onclick],img[class|src|border=0|alt|title|hspace|vspace|width|height|align|onmouseover|onmouseout|name],hr[class|width|size|noshade],span[class|align|style]",
relative_urls : false,
remove_script_host : false,
button_tile_map : true,
safari_warning: false,
width:570,} }],
		['show_toggle_buttons', { Default => 0 }]
    ]), 
	config_template => 'config.tmpl',
	text_filters => {
        'tinymce' => { label => "WYSIWYG", code => sub { $_[0] } }
    },
	callbacks    => {
		'MT::App::CMS::AppTemplateSource.edit_entry' => \&_edit_entry,
		'MT::App::CMS::AppTemplateParam.edit_entry' => \&_edit_entry_param,
	}
}));

# Allows external access to plugin object: MT::Plugin::MyPlugin->instance
sub instance { $plugin; }

sub apply_default_settings {
	my ($plugin, $data, $scope_id) = @_;
	if ($scope_id eq 'system') {
		return $plugin->SUPER::apply_default_settings($data, $scope_id);
	} else {
		my $sys;
		for my $setting (@{$plugin->{'settings'}}) {
			my $key = $setting->[0];
			next if (exists($data->{$key}) && defined($data->{$key}));
			  # don't load system settings unless we need to
			$sys ||= $plugin->get_config_obj('system')->data;
			$data->{$key} = $sys->{$key};
		}
	}
}

sub _edit_entry_param {
	my ($cb, $app, $param, $tmpl) = @_;
	my $blog = $app->blog;
	my $config = $plugin->get_config_hash('blog:'.$blog->id);
	foreach (keys %$config){
		$param->{$_} = $config->{$_};
	}
	require MT::Entry;
	my $convert_breaks;
    if (my $id = $app->param('id')) {
        my $entry = MT::Entry->load($id);
        $convert_breaks = $entry->convert_breaks;
    } else {
        $convert_breaks = $blog->convert_paras;
    }

	if($convert_breaks eq 'tinymce') {
		$param->{load_by_default} = 1;
	}
}

sub _edit_entry {
    my ($cb, $app, $tmpl) = @_;
	my $blog = $app->blog;

	my ($old, $new);
	
	$old = qq{<TMPL_INCLUDE NAME="header.tmpl">};
	$old = quotemeta($old);
    $new = <<HTML;                    
<script type="text/javascript" src="<TMPL_VAR NAME=STATIC_URI>plugins/EnhancedEntryEditing/tiny_mce/tiny_mce.js"></script>
<script type="text/javascript">
<!--
tinyMCE.init({
<TMPL_VAR NAME=TINYMCE_CONFIG>
	mode: "exact",
	elements: "text<TMPL_IF NAME=DISP_PREFS_SHOW_EXTENDED>,text_more</TMPL_IF>",
	oninit: "initWYSIWYG"
});

function initWYSIWYG() {
	<TMPL_IF NAME=LOAD_BY_DEFAULT>
	toggleFormattingButtons('text');
	<TMPL_IF NAME=DISP_PREFS_SHOW_EXTENDED>toggleFormattingButtons('text_more')</TMPL_IF>
	<TMPL_ELSE>
	tinyMCE.execCommand('mceRemoveControl',false,'text');
	<TMPL_IF NAME=DISP_PREFS_SHOW_EXTENDED>tinyMCE.execCommand('mceRemoveControl',false,'text_more');</TMPL_IF>
	</TMPL_IF>
}

function toggleFormattingButtons(t) {
	var d = document.getElementById(t).parentNode.previousSibling.previousSibling || document.getElementById(t).parentNode.previousSibling;
	var imgs = d.getElementsByTagName('img');
	for (var i = 0; i < imgs.length; i++) {
		var img = imgs[i];
		if(img.className == 'wysiwyg-icon') {
			if(img.style.display == 'none')
				img.style.display = 'inline';
			else
				img.style.display = 'none';			
		}
		if(img.style.visibility == 'hidden')
			img.style.visibility = 'visible';
		else
			img.style.visibility = 'hidden';
	}
	<TMPL_UNLESS NAME=SHOW_TOGGLE_BUTTONS>d.style.display = 'none';</TMPL_UNLESS>
}
//-->
</script>
HTML
	$$tmpl =~ s/($old)/$1\n$new\n/;
	foreach (qw( text text_more )) {
		$old = qq{write('<img title="<MT_TRANS phrase="Bold">" onclick="return formatStr(document.entry_form.$_, \\'strong\\')" src="<TMPL_VAR NAME=STATIC_URI>images/formatting-icons/bold.gif" alt="<MT_TRANS phrase="Bold">" width="26" height="19" />');};
		$old = quotemeta($old);
		$new = qq{<TMPL_IF NAME=SHOW_TOGGLE_BUTTONS>
			write('<img class="wysiwyg-icon" title="<MT_TRANS phrase="Enable WYSIWYG Editor">" onclick="tinyMCE.execCommand(\\'mceAddControl\\',false,\\'$_\\'); toggleFormattingButtons(\\'$_\\');" src="<TMPL_VAR NAME=STATIC_URI>plugins/EnhancedEntryEditing/images/add.gif" alt="<MT_TRANS phrase="Enable WYSIWYG Editor">" width="26" height="19" />');
			write('<img class="wysiwyg-icon" title="<MT_TRANS phrase="Disable WYSIWYG Editor">" onclick="tinyMCE.execCommand(\\'mceRemoveControl\\',false,\\'$_\\'); toggleFormattingButtons(\\'$_\\');" src="<TMPL_VAR NAME=STATIC_URI>plugins/EnhancedEntryEditing/images/remove.gif" alt="<MT_TRANS phrase="Disable WYSIWYG Editor">" style="display:none; visibility:hidden;" width="26" height="19" />');
			</TMPL_IF>};
		$$tmpl =~ s/($old)/$new\n$1\n/;
	}

}

1;